#version 430


layout (location=0) in vec3 vertexPosition;
layout (location=1) in vec3 vertexNormal;
layout (location=2) in vec2 vertexUV;
layout (location=3) in vec3 vertexTangent;
out vec3 normal;
out vec2 uv;
out vec3 tangent;
out vec4 origPos;
out float mak;

uniform mat4 projectionMatrix;
uniform mat4 modelViewMatrix;

uniform float g_time;


vec4 rotateXZ(vec4 p, float a) {
  vec4 r = p;
  r.x = cos(a)*p.x - sin(a)*p.z;
  r.z = sin(a)*p.x + cos(a)*p.z;
  return r;
}

vec3 rotateXZ3(vec3 p, float a) {
  return rotateXZ(vec4(p, 0.0), a).xyz;
}

vec4 rotateXY(vec4 p, float a) {
  vec4 r = p;
  r.x = cos(a)*p.x - sin(a)*p.y;
  r.y = sin(a)*p.x + cos(a)*p.y;
  return r;
}

vec3 rotateXY3(vec3 p, float a) {
  vec3 r = p;
  r.x = cos(a)*p.x - sin(a)*p.y;
  r.y = sin(a)*p.x + cos(a)*p.y;
  return r;
}

vec4 rotateYZ(vec4 p, float a) {
  vec4 r = p;
  r.y = cos(a)*p.y - sin(a)*p.z;
  r.z = sin(a)*p.y + cos(a)*p.z;
  return r;
}
vec3 rotateYZ3(vec3 p, float a) {
  vec3 r = p;
  r.y = cos(a)*p.y - sin(a)*p.z;
  r.z = sin(a)*p.y + cos(a)*p.z;
  return r;
}

layout(binding=1) uniform sampler2D texNoise;

float noise(in vec3 x) //3d noise from iq
{
    vec3 p = floor(x);
    vec3 f = fract(x);
        f = f*f*(3.0-2.0*f);
        vec2 uv = (p.xy+vec2(37.0,17.0)*p.z) + f.xy;
        vec2 rg = texture2D( texNoise, (uv+ 0.5)/256.0, -100.0 ).yx;
        return mix( rg.x, rg.y, f.z );
}

float fbm(in vec3 x)
{
    float rz = 0.;
    float a = .35;
    for (int i = 0; i<4; i++)
    {
        rz += noise(x)*a;
        a*=.5;
        x*= 4.;
    }
    return rz;
}


uniform float g_makkara = 0.0;
uniform float g_wave = 0.0;

void main() {
  gl_Position = vec4(vertexPosition*0.80, 1.0);

  vec4 posse = gl_Position*1.0;

  origPos = posse*1.0;

 
  float times = g_time*0.001;
  float times2 = g_time*0.002;

  normal = vertexNormal;
  tangent = vertexTangent;
 // gl_Position.xyz += normal*1.0*(1.0+sin(posse.y*0.4+g_time*3.4));


 // float makkara = smoothstep(0.0, 1.0, clamp(0.50+1.50*sin(g_time*0.2), 0.0, 1.0))*g_makkara;

  float makkara = g_makkara*1.0;

 // makkara = 1.0;

  float yk = 1.20;

  float noiseFreq = 2.08;
  float nois1 = fbm(origPos.xyz*0.230*noiseFreq+3.20*vec3(g_wave, g_wave*1.2, 0.3));
  float nois2 = fbm(origPos.yzx*0.30*noiseFreq+2.70*vec3(g_wave*0.3, g_wave*0.62, 0.3));
  float nois3 = fbm(origPos.zyx*0.20*noiseFreq+1.360*vec3(g_wave*0.1, g_wave*0.12, 0.3));

  gl_Position.xyz += nois1*normal.xyz;

  vec4 pos1 = rotateXZ(gl_Position, -g_time*0.0+yk*posse.y*sin(times2)+0.0*posse.z);
  vec4 pos2 = rotateXZ(gl_Position, -(g_time+0.1)*0.0+yk*posse.y*sin(times2+0.02)+0.0*posse.z);

  origPos = pos2-pos1;

  gl_Position = rotateXZ(gl_Position, -g_time*0.0+yk*posse.y*sin(times2)+0.0*posse.z);

  gl_Position.z += posse.y*5.0;

  gl_Position.xz *= mix(1.0, 0.15, makkara);
  gl_Position.y *= (1.0+makkara*1.0);
  normal = rotateXZ3(normal, -g_time*0.0+yk*posse.y*sin(times2)+0.0*posse.z);

  normal.z += posse.y*1.0;

  tangent = rotateXZ3(tangent, -g_time*0.0+yk*posse.y*sin(times2)+0.0*posse.z);

//  gl_Position.xz *= 1.0+sin(cos(posse.y+g_time*2.0)+g_time*9.2)*0.02;

 // normal = (modelViewMatrix * vec4(normal, 0.0)).xyz;

    vec4 posNR = gl_Position;
    vec3 normalNR = normal;
    vec3 tangentNR = tangent;

    float mk = 1.0;

   gl_Position = rotateXY(gl_Position, g_time*0.0+mk*posse.y*(cos(times)))+0.10*rotateYZ(gl_Position, 0.01*posse.y*(cos(times)));
   normal = rotateXY3(normal, g_time*0.0+mk*posse.y*(cos(times)))+0.10*rotateYZ3(normal, 0.01*posse.y*(cos(times)));
   tangent = rotateXY3(tangent, g_time*0.0+mk*posse.y*(cos(times)))+0.10*rotateYZ3(tangent, 0.01*posse.y*(cos(times)));
//   gl_Position = 2.0*rotateYZ(gl_Position, 0.0030*posse.y*(cos(times)));


   gl_Position = mix(posNR, gl_Position, makkara);
   normal = mix(normalNR, normal, makkara);
   tangent = mix(tangentNR, tangent, makkara);


   tangent = (modelViewMatrix * vec4(tangent, 0.0)).xyz;

  // gl_Position.y *= 3.0;
/*
  vec3 lightPos = vec3(10*cos(g_time*2.0), 0.0, -5.0);
  vec3 surfPos = vec3(0.0f, 0.0f, 0.0f);
  vec3 lightDir = lightPos-surfPos;

  float lightDist = sqrt(dot(lightDir, lightDir));

  float normalLen = sqrt(dot(normal, normal));

  float d = dot(lightDir, normal)/(lightDist*normalLen);
  d = clamp(-d, 0.0, 1.0);

  gl_Position.xyz += 2.0*vertexNormal*pow(d,12.0f);
*/

 // gl_Position = projectionMatrix * modelViewMatrix * gl_Position;
 // normal = vertexNormal;
 // tangent = vertexTangent;

  uv = vertexUV;


  uv = (posse.xy*0.10+vec2(posse.z*0.1-posse.x*0.05, posse.x*0.02-posse.y*0.06))*10.0;

  mak = makkara;

 // gl_Position.z += 0.9350;
}
